#!/usr/bin/env python
# -*- coding:UTF-8 -*-
"""
    This file will control progress under a g-code for Gmoccapy interface 
        for LinuxCNC machines

    Copyright (C) 2014 Serguei Glavatski, gsngla@gmail.com
    

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""

import hal                  # base hal class to react to hal signals
import hal_glib             # needed to make our own hal pins
import gtk                  # base for pygtk widgets and constants
import sys                  # handle system calls
import linuxcnc             # to get our own error sytsem
import gobject              # needed to add the timer for periodic

work_thread = 500            # work_thread means how often pins will be updated (ms)

from gmoccapy import preferences
from gmoccapy import getiniinfo
from subprocess import Popen, PIPE

class ProgressClass:

    def __init__(self, halcomp, builder, useropts):
        self.builder = builder
        get_ini_info = getiniinfo.GetIniInfo()
        prefs = preferences.preferences(get_ini_info.get_preference_file_path())
        theme_name = prefs.getpref("gtk_theme", "Follow System Theme", str)
        if theme_name == "Follow System Theme":
            theme_name = gtk.settings_get_default().get_property("gtk-theme-name")
        gtk.settings_get_default().set_string_property("gtk-theme-name", theme_name, "")
        self.stat = linuxcnc.stat()
        # get all widgets and connect them
        self.progressbar = self.builder.get_object("progress_bar")
        self.file_lines = 0 
        self.current_file = ""
        self.timeout_id = gobject.timeout_add(work_thread, self._update_reald) # time between calls to the function, in milliseconds
	#  timer tracking function        
    def _update_reald( self ):
        self.stat.poll() 
        if self.stat.task_state == 4 : #linuxcnc.STATE_ON
			if self.stat.task_mode == 2 : #linuxcnc.MODE_AUTO
				if self.current_file != self.stat.file: # g-code file is changed
       				 try:
       				 	with open(self.stat.file,"r") as f:
       				 		#get number of lines in g-code file (without create an additional list in memory)
       				 		self.file_lines = sum(1 for _ in f)
       				 	# save new file name	            
       				 	self.current_file = self.stat.file 
       				 	print "open file:", self.stat.file, " number of lines:", self.file_lines
       				 	f.close()
       				 except : 
       				 	print "Open ",self.stat.file," error!"       				
				a= int(Popen('halcmd getp motion.program-line ', shell=True, stdout=PIPE).stdout.read())
				self.progressbar.set_fraction(1.000*a/self.file_lines) # current g-code line complete in %
				#print "STATE:",a, self.file_lines, 1.000*a/self.file_lines, self.progressbar.get_fraction()
			else :
		        	self.progressbar.set_fraction(0)
        else :
             	self.progressbar.set_fraction(0)
        return True
                                                    
def get_handlers(halcomp,builder,useropts):
    return(ProgressClass(halcomp,builder,useropts))

